/**
 * @author Christian Brel <christian@the6thscreen.fr, ch.brel@gmail.com>
 */

/// <reference path="../utils/Logger.ts" />
/// <reference path="./ViewportManager.ts" />

declare var $: any; // Use 'JQuery' lib
declare var io: any; // Use 'Socket.IO' lib

class SocketManager {

    /**
     * The Paint Server's URL.
     *
     * @property _serverURL
     * @type string
     */
    private _serverURL:string;

    /**
     * The Paint Server socket.
     *
     * @property _socket
     * @type any
     */
    private _socket:any;

    private _vpm : ViewportManager;

    /**
     * Constructor.
     *
     * @constructor
     */
    constructor() {
        this._socket = null;
    }

    /**
     * Set ViewportManager.
     *
     * @method setViewportManager
     * @param {ViewportManager} viewportManager - The ViewportManager.
     */
    setViewportManager(viewportManager:ViewportManager) {
        this._vpm = viewportManager;
    }

    /**
     * Connection to server.
     *
     * @method _connection
     * @private
     */
    private _connection() {
        Logger.debug("Connection to server : " + this._serverURL);
        var self = this;

        this._socket = io(this._serverURL,
            {"reconnection" : true, 'reconnectionAttempts' : 10, "reconnectionDelay" : 1000, "reconnectionDelayMax" : 5000, "timeout" : 5000, "autoConnect" : true});

        this._listen();

        this._socket.on("connect", function () {
            Logger.info("Connected to Paint Server.");
            self._vpm.hideConnectionMenu();
        });

        this._socket.on("error", function (errorData) {
            Logger.error("An error occurred during connection to Paint Server.");
        });

        this._socket.on("disconnect", function () {
            Logger.info("Disconnected to Paint Server.");
        });

        this._socket.on("reconnect", function (attemptNumber) {
            Logger.info("Connected to Paint Server after " + attemptNumber + " attempts.");
        });

        this._socket.on("reconnect_attempt", function () {
            Logger.info("Trying to reconnect to Paint Server.");
        });

        this._socket.on("reconnecting", function (attemptNumber) {
            Logger.info("Trying to connect to Paint Server - Attempt number " + attemptNumber + ".");
        });

        this._socket.on("reconnect_error", function (errorData) {
            Logger.error("An error occurred during reconnection to Paint Server.");
        });

        this._socket.on("reconnect_failed", function () {
            Logger.error("Failed to connect to Paint Server. No new attempt will be done.");
            self._vpm.showConnectionMenu();
        });
    }

    /**
     * Listen for Server messages.
     *
     * @method _listen
     * @private
     */
    private _listen() {
        Logger.debug("listen");
        var self = this;

        this._socket.on("viewport", function(viewportDescription) {
            self.viewportDescriptionProcess(viewportDescription);
        });
    }

    /**
     * Set Server Url.
     *
     * @method setServerUrl
     * @param {string} serverURL - The Server's URL.
     */
    setServerUrl(serverURL:string) {
        this._serverURL = serverURL;
        this._connection();
    }

    /**
     * Set Profil.
     *
     * @method setProfil
     * @param {string} username - The Username.
     * @param {number} width - The Screen's width.
     * @param {number} height - The Screen's height.
     */
    setProfil(username: string, width: number, height: number) {
        this._vpm.setUsername(username);

        $('#main_content').width(width);
        $('#main_content').height(height);
        this._socket.emit("profil", {"username" : username, "width" : width, "height" : height});
    }

    /**
     * Set Username.
     *
     * @method setUsername
     * @param {string} username - The Username.
     */
    setUsername(username: string) {
        this._vpm.setUsername(username);
        this._socket.emit("username", {"username" : username});
    }

    /**
     * Process Viewport Description.
     *
     * @method viewportDescriptionProcess
     * @param {any} viewportDescription - The Viewport Description.
     */
    viewportDescriptionProcess(viewportDescription : any) {
        this._vpm.setWidth(viewportDescription.width);
        this._vpm.setHeight(viewportDescription.height);
        this._vpm.showViewport();
    }
}